#include "StdAfx.h"
#include "WASGetNewRoom.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"

CWASGetNewRoom::CWASGetNewRoom(CWASActor *pActor) : CWASCommand(pActor)
{
	m_sRoomInfo.Clear();
	m_wsUserID.clear();
}

CWASGetNewRoom::~CWASGetNewRoom(void)
{

}

void CWASGetNewRoom::ExecuteAsync()
{
	USES_UTF8_CONVERSION;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	m_sRoomInfo.Clear();
	char szOSType[100] = {0,};
	
	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/chat/GetNewRoom.php");

	sprintf(szOSType, "%d", pMessengerActor->GetOSType());

	m_httpFacade.session().SetURL(W2U8(strURL.c_str()),	pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);
	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));
	m_httpFacade.session().AddBodyVariable("username", W2U8(m_wsUserName.c_str()));

	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());	
	m_httpFacade.session().AddBodyVariable("ostype", szOSType);

	if (m_wsChatBuddyID.empty() == false)
	{
		m_httpFacade.session().AddBodyVariable("target_user_id", W2U8(m_wsChatBuddyID.c_str()));
		m_httpFacade.session().AddBodyVariable("target_user_name", W2U8(m_wsChatBuddyName.c_str()));
	}

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	
	m_httpFacade.StopRequest();
	m_httpFacade.SendRequest(this);
}

bool CWASGetNewRoom::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	if(pDom == NULL)
		return false;

	do
	{
		USES_UTF8_CONVERSION;

		nfc::IIkXMLNode *pNode, *pChildNode;
		nfc::IIkXMLAttribute * pAttr;		
			

		//------------
		/*  ȣ 
		<room id='100'/>
		*/
		pNode = pDom->GetdocumentElement()->selectSingleNode("room");		
		if(pNode == NULL)
			break;

		pAttr = pNode->getAttribute("id");
		if(pAttr)
		{
			m_sRoomInfo.nRoomID = atoi(pAttr->Gettext().c_str());
		}

		pChildNode = pNode->selectSingleNode("server");		
		if(pChildNode == NULL)
			break;		
		
		pAttr = pChildNode->getAttribute("address");
		if(pAttr)
		{			
			m_sRoomInfo.strServerAddress = pAttr->Gettext().c_str();			
		}

		pAttr = pChildNode->getAttribute("port");
		if(pAttr)
		{			
			m_sRoomInfo.nServerPort = atoi(pAttr->Gettext().c_str());			
		}

		pAttr = pChildNode->getAttribute("type");
		if(pAttr)
		{			
			m_sRoomInfo.strRoomType = pAttr->Gettext().c_str();			
		}

	} while(FALSE);

	return true;
}


// override in drived class
void CWASGetNewRoom::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}
	
	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	GT_ROOM_DATA sData;
	m_sRoomInfo.Convert(sData);

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseGetNewRoom(m_pActor, GetID(), m_eError, sData) == false)
		{
			break;
		}
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->DeleteCommand(GetID());
}


void CWASGetNewRoom::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	GT_ROOM_DATA sData;
	m_sRoomInfo.Convert(sData);

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseGetNewRoom(m_pActor, GetID(), m_eError, sData) == false)
		{
			break;
		}
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->DeleteCommand(GetID());
}